class Emote(object):
# see if copilot is good at using right emojis for emotions

    '''
    adoration = "😘"
    agressive = "😠"
    amazed = "🤩"
    amorous = "😍"	
    appreciative = "😘"	
    # 😘	brave
    # 😘	calm
    # 😘	cheerful
    # 😘	childlike
    # 😘	clever
    # 😘	comfortable
    # 😘	confident
    # 😘	courageous
    🤣	joy	amused
    😡	angry
    😒	annoyed
    😰	anxious
    # 😢	ashamed
    # 😣	awesome
    😈	aroused
    # 😔	awkward
    # 😪	bored
    # 😭	brave
    😧	cautious
    😁	cheerful
    🥰	affectionate
    🥰	compassionate
    😕	confused
    # 😨	cranky
    # 😴	determined
    # 😌	eager
    👿	contemptful
    😎	cool
    # 😏	curious
    😇	delighted
    😭	depressed
    🥺	desperate
    😞	disappointed
    # 😓	dismayed
    🤢	disgusted
    😦	disturbed
    😳	embarrassed
    # 😮	excited
    😀	happy
    😊	hopeful
    😱	horrified
    😤	hostile
    😍	love
    😃	optimistic
    😓	pessimistic
    😢	pity
    😣	powerless
    😄	proud
    # 😜	relaxed
    # 😝	relieved
    # 😒	sad
    # 😩	sarcastic
    # 😵	scared
    🤬	rageful
    😏	satisfied
    😎	surprised
    😴	tired
    😌	unamused
    # 😛	understanding
    😜	randy
    😫	rejected
    😌	relieved
    🙁	sad
    😨	scared
    😮	shocked
    🤮	sick
    🤓	smart
    # 😒 stupid
    😲	suprised
    🤔	suspicious
    🤗	sympathy
    😴	tired
    😇	victorious
    # 😟	worried
    # 😔	worried
    '''


    # def get_emoji(feeling):
    #     """[
    #         returns an emoji for a given feeling or emotion
    #         a feeling is usually an adjective
    #     ]
    #     Args:
    #         feeling ([type]): [description]
    #     """
    #     if feeling == "adoration":
    #         return "😘"
    #     elif feeling == "agressive":
    #         return "😠"
    #     elif feeling == "amusement":
    #         return "😂"
    #     elif feeling == "anger":
    #         return "😡"
    #     elif feeling == "anguished":
    #         return "😧"
    #     elif feeling == "anticipation":
    #         return "😜"
    #     elif feeling == "apathy":
    #         return "😐"
    #     elif feeling == "apprehension":
    #         return "😨"
    #     elif feeling == "arrogance":
    #         return "😎"
    #     elif feeling == "ashamed":
    #         return "😞"
    #     elif feeling == "awe":
    #         return "😲"
    #     elif feeling == "awkwardness":
    #         return "😕"
    #     elif feeling == "boredom":
    #         return "😒"
    #     elif feeling == "brave":
    #         return "🙌"
    #     elif feeling == "calm":
    #         return "🌞"
    #     elif feeling == "cautiousness":
    #         return "😯"
    #     elif feeling == "contentment":
    #         return "😀"
    #     elif feeling == "contempt":
    #         return "😒"
    #     elif feeling == "contemptuousness":
    #         return "😞"
    #     elif feeling == "courage":
    #         return "🌱"
    #     elif feeling == "courageousness":
    #         return "😲"
    #     elif feeling == "creativity":
    #         return "💥"
    #     elif feeling == "crestfallen":
    #         return "😞"
    #     elif feeling == "curiosity":
    #         return "😜"
    #     elif feeling == "daze":
    #         return "😕"
    #     elif feeling == "delight":
    #         return "😀"
    #     elif feeling == "depression":
    #         return "😞"
    #     elif feeling == "despair":
    #         return "😢"
    #     elif feeling == "despairing":
    #         return "😢"
    #     elif feeling == "despairingly":
    #         return "😢"
    #     elif feeling == "despise":
    #         return "😞"
    #     elif feeling == "despondence":
    #         return "😞"
    #     elif feeling == "despondency":
    #         return "😞"
    #     elif feeling == "despondent":
    #         return "😞"
    #     elif feeling == "despondently":
    #         return "😞"
    #     elif feeling == "determined":
    #         return "😛"
    #     elif feeling == "doubt":
    #         return "😖"
    #     elif feeling == "doubtful":
    #         return "😕"
    #     elif feeling == "doubtfully":
    #         return "😕"
    #     elif feeling == "doubtfulness":
    #         return "😕"
    #     elif feeling == "dread":
